//#define NU32_STANDALONE  // uncomment if program is standalone, not bootloaded
#include "NU32.h"          // plib.h, config bits, constants, funcs for startup and UART
#include "TCS3771.h"

void init_I2C(void);
void TCS_init(void);
int TCS_checkID(void);
int TCS_readDist(void);
void TCS_I2Cwrite(int address, int command, int data);

int main(void) {
  NU32_Startup(); // cache on, min flash wait, interrupts on, LED/button init, UART init

  init_I2C();
  if (!TCS_checkID()) {
    while (1); // ID not correct, something wrong with communication or chip
  }

  TCS_init();

  unsigned int prox = 0;
  char message[50];

  while (1) {
    WriteCoreTimer(0);
    prox = TCS_readDist();
    
    sprintf(message,"%d\r\n",prox);
    NU32_WriteUART1(message);

    NU32LED1 = !NU32LED1; // toggle the LEDs
    NU32LED2 = !NU32LED2;

    while (ReadCoreTimer() < 40000000 / 50) {
    }
  }
  return 0;
}

void init_I2C(void) {
  // set BRG
  I2C1BRG = 90; // 400kHz

  // turn on module
  I2C1CONbits.ON = 1;
}

void TCS_init() {
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_ENABLE, 0x00); // put chip to sleep

  // write CONTROL = 0b00010010 = 0x12 = 18dec (AGAIN = 16x, use clear (IR) diode, 100mA IR pulses)
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_CONTROL, 0b11110010); // 18dec   0xA0 for 25mA, red(ir) diode, 1x gain
  // write ATIME = 0xFF (2.4ms RGBC integration time)
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_ATIME, 0xFF); // 255dec
  // write PTIME = 0xFF (2.4ms proximity integration time)
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_PTIME, 0xFF); // 255dec
  // write PPCOUNT = 0x40 (64 pulses = 896us)
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_PPCOUNT, 2); // 64dec
  // write WTIME = 0xFF (2.4ms wait time)
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_WTIME, 0xFF); // 255dec
  // write ENABLE = 0b00000111 = 0x0F (PON AEN PEN = 1 WEN = 0)
  TCS_I2Cwrite(TCS_ADDRESS_WRITE, TCS_ENABLE, 0x0F); // 15dec
}

int TCS_checkID(void) {
  char id = 0;
  StartI2C1(); //Send line start condition
  IdleI2C1(); //Wait to complete
  MasterWriteI2C1(TCS_ADDRESS_WRITE); //Write out slave address OR 1 (read command)
  IdleI2C1(); //Wait to complete
  MasterWriteI2C1(0b10000000 | TCS_ID); //Write out slave address OR 1 (read command)
  IdleI2C1(); //Wait to complete
  RestartI2C1();
  IdleI2C1();
  MasterWriteI2C1(TCS_ADDRESS_READ);
  id = MasterReadI2C1(); //Read in a value
  StopI2C1(); //Send line stop condition
  IdleI2C1(); //Wait co complete

  if (id == 25) {
    return 1; // right chip
  } else {
    return 0; // wrong chip
  }
}

void TCS_I2Cwrite(int address, int command, int data) {
  StartI2C1(); //Send the Start Bit
  IdleI2C1(); //Wait to complete

  MasterWriteI2C1(address); //Sends the slave address over the I2C line.  This must happen first so the
  //proper slave is selected to receive data.
  IdleI2C1(); //Wait to complete

  MasterWriteI2C1(0b10000000 | command); //Sends data byte over I2C line
  IdleI2C1(); //Wait to complete

  MasterWriteI2C1(data); //Sends data byte over I2C line
  IdleI2C1(); //Wait to complete

  StopI2C1(); //Send the Stop condition
  IdleI2C1(); //Wait to complete
}

int TCS_readDist(void) {
  char data[2];
  StartI2C1(); //Send line start condition
  IdleI2C1(); //Wait to complete
  MasterWriteI2C1(TCS_ADDRESS_WRITE); //Write out slave address OR 1 (read command)
  IdleI2C1(); //Wait to complete
  MasterWriteI2C1(0b10100000 | TCS_PDATA); //Write out slave address OR 1 (read command)
  IdleI2C1(); //Wait to complete
  RestartI2C1();
  IdleI2C1();
  MasterWriteI2C1(TCS_ADDRESS_READ);
  MastergetsI2C1(2, data, 10000000);
  StopI2C1(); //Send line stop condition
  IdleI2C1(); //Wait co complete

  StartI2C1(); //Send line start condition
  IdleI2C1(); //Wait to complete
  MasterWriteI2C1(TCS_ADDRESS_WRITE); //Write out slave address OR 1 (read command)
  IdleI2C1(); //Wait to complete
  MasterWriteI2C1(0xE7); //clear interrupts
  IdleI2C1(); //Wait to complete
  StopI2C1(); //Send line stop condition
  IdleI2C1(); //Wait co complete

  int i = 0;
  unsigned int dist;
  if (data[1] == 0) {
    dist = data[0]&0xFF;
  } else {
    dist = ((data[i + 1]&0xFF) << 8) | (data[i]&0xFF);
  }

  return dist;
}

